;;; 
;;; book-code.ss
;;; David Evans
;;; 6 October 2009

;;; This file contains some procedures that are useful for Problem Set 5.
;;; (Note all of the procedures are defined in the course book.)

;;; We use assert to check properties that must be true.  If an assertion fails,
;;; it probably means there is a bug in your code.  This is a form of defensive
;;; programming.

(define (assert pred)
  (if (not pred) (error "Assertion failed!") 0))

;;; Selected mutable List procedures from Chapter 9.

(define (mlist-append! p q)
  (if (null? p) (error "Cannot append! to an empty list.")
      (if (null? (mcdr p)) (set-mcdr! p q)
          (mlist-append! (mcdr p) q))))

;;; Note that this is different from mlist-filter! in the book! 
;;; Instead of mutating a list, it takes a mutable list as input
;;; and produces a new mutable list.

(define (mlist-filter test p)
  (if (null? p) null
      (if (test (mcar p))
          (mcons (mcar p) (mlist-filter test (mcdr p)))
          (mlist-filter test (mcdr p)))))

;;; Similar to list-get-element
(define (mlist-get-element p n) 
   (if (null? p)
       (error "Index out of range")
       (if (= n 1)	
           (mcar p)	
           (mlist-get-element (mcdr p) (- n 1)))))

;;; Similar to list-find-best but for mlists:

(define (pick-better cf p1 p2)
  (if (cf p1 p2) p1 p2))

(define (mlist-find-best cf p)
  (if (null? (mcdr p)) (mcar p)
      (pick-better cf (mcar p) (mlist-find-best cf (mcdr p)))))

;;; Same as list-sum, but for mlist
(define (mlist-sum p)
  (if (null? p)
      0
      (+ (mcar p) (mlist-sum (mcdr p)))))

;;; Tagged Lists (from Section 5.6)

(define (make-tlist tag p) (cons tag p))
(define (tlist-get-tag p) (car p))

(define (tlist-get-data tag p) 
  (if (not (eq? (tlist-get-tag p) tag))
      (error (format "Bad tag: ~a (expected ~a)" (tlist-get-tag p) tag))
      (cdr p)))

(define (tlist-get-element tag p n)
  (if (eq? (tlist-get-tag p) tag)
      (list-get-element (cdr p) n)
      (error (format "Bad tag: ~a (expected ~a)" (tlist-get-tag p) tag))))

(define (list-find-element p el) ; evaluates to the index of element el
  (define (list-find-element-helper p el n)
    (if (null? p)
        0 ; element not found
        (if (eq? (car p) el)
            n
            (list-find-element-helper (cdr p) el (+ n 1)))))
  (list-find-element-helper p el 1))

;;;
;;; List procedures from Chapter 5.
;;;

(define (list-get-element p n) ;; first element is 1
   (if (null? p)
       (error "Index out of range")
       (if (= n 1)	
           (car p)	
           (list-get-element (cdr p) (- n 1)))))

(define (list-reverse p)
  (if (null? p) 
      null 
      (list-append (list-reverse (cdr p)) (list (car p)))))

(define (intsto n)
  (if (= n 0) 
      null 
      (list-append (intsto (- n 1)) (list n))))

